import json
from jsonschema import RefResolver, Draft7Validator


def validate_request(request_body):
    channelpayload = {
        "$id": "/schemas/channelpayload",
        "type": "object",
        "properties": {
            "recipient": {"type": "string", "minLength": 1, "maxLength": 100},
            "personalized_attributes": {"type": "object"}
        },
        "required": ["recipient"]
    }
    payloads = {
        "$id": "/schemas/payloads",
        "type": "object",
        "propertyNames": {
            "enum": [
                "EMAIL",
                "SMS"
            ]
        },
        "patternProperties": {
            "": {
                "$ref": "/schemas/channelpayload"
            }
        },
        "minProperties": 1
    }
    request = {
        "$id": "/schemas/request",
        "type": "object",
        "properties": {
            "alert_id": {
                "type": "string",
                "minLength": 24,
                "maxLength": 24,
                "pattern": "^[A-Fa-f0-9]{24}$"
            },
            "user_id": {"type": "string"},
            "transaction_id": {
                "type": "string",
                "minLength": 1,
                "maxLength": 50
            },
            "payloads": {"$ref": "/schemas/payloads"}
        },
        "required": ["alert_id", "transaction_id", "payloads"]
    }
    channelpayload_schema = json.loads(json.dumps(channelpayload))
    payloads_schema = json.loads(json.dumps(payloads))
    request_schema = json.loads(json.dumps(request))
    schema_store = {
        channelpayload_schema['$id']: channelpayload_schema,
        payloads_schema['$id']: payloads_schema,
        request_schema['$id']: request_schema,
    }
    resolver = RefResolver.from_schema(request_schema, store=schema_store)
    validator = Draft7Validator(request_schema, resolver=resolver)
    validator.validate(request_body)



import json
import urllib3

from base64 import b64encode

from informclient.utils.api_description import InformClientRoutes
from informclient.api.request.request_validator import validate_request


class InformClient(object):

    def __init__(self, base_url, app_id, api_secret):
        """
        Instantiate a new Inform client.
        Args:
          base_url (str):
          app_id (str):
          api_secret (str):
        """
        if base_url == '' or app_id == '' or api_secret == '':
            raise ValueError("base_url or app_id or api_secret is empty")
        self.base_url = base_url
        credentials = b64encode('{}:{}'.format(app_id, api_secret).encode())
        self.headers = {
            'Content-Type': 'application/json',
            'Authorization': 'Basic {}'.format(credentials.decode()),
            'MOE-APPKEY': app_id
        }

    def send_alert(self, request_body):
        """
        Send alert to provided recipients
        Args:
            request_body:
        Returns:
            response: response of the Inform API
        """

        validate_request(request_body)

        url = "%s/%s" % (self.base_url, InformClientRoutes.INFORM_SEND)

        http = urllib3.PoolManager()
        encoded_body = json.dumps(request_body).encode('utf-8')
        resp = http.request("POST", url, body=encoded_body,
                            headers=self.headers, timeout=10, retries=3)
        response_data = json.loads(resp.data.decode("utf-8"))
        response_data.update({"status_code": resp.status})
        return response_data



from setuptools import setup

long_description = ''
with open('README.md', 'r') as fh:
    long_description += fh.read()

with open('LICENSE', 'r') as fh:
    long_description += fh.read()


setup(
    name='moe-inform-client',
    version='0.0.2',
    author='MoEngage',
    author_email='inform@moengage.com',
    description='This Python Package helps you in sending notifications through MoEngage Inform',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/moengage/inform-client-python',
    project_urls={
            'Bug Tracker': 'https://github.com/moengage/inform-client-python/issues',
    },
    install_requires=['StrEnum~=0.4.9',
                      'urllib3~=1.26.5',
                      'jsonschema~=4.17.3'
                      ],
    keywords=['MoEngage', 'Inform', 'MoEngage Inform', 'Customer Engagement', 'Transactional Alerts'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
    ],
    py_modules=['informclient.informclient',
                'informclient.utils.api_description',
                'informclient.api.request.request_validator'],
    python_requires='>=3.9'
)
